Class: Sqlbible
- Inherits:
-
Object
- Object
- Sqlbible
- Defined in:
- lib/sqlbible.rb,
lib/sqlbible/convert.rb,
lib/sqlbible/version.rb
Defined Under Namespace
Classes: Verse
Constant Summary collapse
- VERSION =
'1.3.3'
Class Attribute Summary collapse
-
.bibles_dir ⇒ Object
readonly
Returns the value of attribute bibles_dir.
Instance Attribute Summary collapse
-
#db_filename ⇒ Object
readonly
Returns the value of attribute db_filename.
Class Method Summary collapse
-
.bible_names ⇒ Object
Get a list of the names of the bibles in bibles_dir.
-
.convert(osis_fn, sqlite_fn) ⇒ Object
Convert a bible file in OSIS XML format to another file in SQLite format for Sqlbible.
-
.db_schema ⇒ Object
Get the database schema as string.
- .resolve_db_filename(db_name) ⇒ Object
-
.schema_major_minor ⇒ Object
Get an array of the major and minor version of the database schema of the actual version of the lib.
-
.schema_version ⇒ Object
Get the version of the database schema of the actual version of the lib.
-
.select_schema_major_minor(db) ⇒ Object
Get an array of the major and minor version of an Sqlbible database.
-
.select_schema_version(db) ⇒ Object
Get the version of an Sqlbible database.
Instance Method Summary collapse
-
#initialize(db_name) ⇒ Sqlbible
constructor
A new instance of Sqlbible.
-
#passage(pass) ⇒ Object
Get an array of Verse instances for a Scripref::Passage.
-
#reference(ref) ⇒ Object
Get an array of arrays of Verse instances for a Scripref reference (array of Scripref::Passage instances).
-
#schema_version ⇒ Object
Get the schema version of the SQLite database of the instance.
-
#search(*searches, range: nil) ⇒ Object
Get an array of Verse instances for a search with one ore more regular expressions (optional a search range of a Scripref reference is possible).
Constructor Details
#initialize(db_name) ⇒ Sqlbible
Returns a new instance of Sqlbible.
14 15 16 17 18 19 20 21 22 23 24 |
# File 'lib/sqlbible.rb', line 14 def initialize db_name @db_filename = Sqlbible.resolve_db_filename(db_name) unless File.exist? @db_filename fail ArgumentError, format('database %s not found, file %s does not exist', db_name, @db_filename) end @db = SQLite3::Database.new(@db_filename) @db.enable_load_extension(true) pcre_extension_file = ENV['SQLITE_PCRE_EXTENSION'] @db.load_extension(pcre_extension_file) if pcre_extension_file check_schema_version end |
Class Attribute Details
.bibles_dir ⇒ Object (readonly)
Returns the value of attribute bibles_dir.
80 81 82 |
# File 'lib/sqlbible.rb', line 80 def bibles_dir @bibles_dir end |
Instance Attribute Details
#db_filename ⇒ Object (readonly)
Returns the value of attribute db_filename.
12 13 14 |
# File 'lib/sqlbible.rb', line 12 def db_filename @db_filename end |
Class Method Details
.bible_names ⇒ Object
Get a list of the names of the bibles in bibles_dir
88 89 90 |
# File 'lib/sqlbible.rb', line 88 def bible_names Dir[File.join(bibles_dir, '*.sqlbible')].map {|fn| File.basename(fn, '.sqlbible')}.sort end |
.convert(osis_fn, sqlite_fn) ⇒ Object
Convert a bible file in OSIS XML format to another file in SQLite format for Sqlbible
13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 |
# File 'lib/sqlbible/convert.rb', line 13 def self.convert osis_fn, sqlite_fn last_bookid = nil booknumber = 0 doc = Nokogiri.XML(File.read(osis_fn)) db = SQLite3::Database.new(sqlite_fn) db.execute_batch(Sqlbible.db_schema) doc.css('verse').each do |v| osisid = v['osisID'] bookid, chapter, verse = osisid.split('.') unless bookid == last_bookid booknumber += 1 db.execute "insert into books (bookid, booknumber) values (?, ?)", [bookid, booknumber] last_bookid = bookid end xml = v.children.to_xml plaintext = '' v.children.each do |e| case e.name when 'note' # ignore when 'divineName' plaintext << e.text.upcase when 'lb' plaintext << "\n" else plaintext << e.text end end plaintext.strip! db.execute "insert into text (osisid, bookid, booknumber, chapter, verse, xml, plaintext) values (?, ?, ?, ?, ?, ?, ?)", [osisid, bookid, booknumber, chapter, verse, xml, plaintext] end end |
.db_schema ⇒ Object
Get the database schema as string
83 84 85 |
# File 'lib/sqlbible.rb', line 83 def db_schema File.read(File.join(__dir__, '../schema.sql')) end |
.resolve_db_filename(db_name) ⇒ Object
115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 |
# File 'lib/sqlbible.rb', line 115 def resolve_db_filename db_name db_name = db_name.to_s if File.basename(db_name, '.sqlbible') == db_name fn = File.join(bibles_dir, format('%s.sqlbible', db_name)) unless bible_names.include?(db_name) msg = format('database %s not found, file %s does not exist', db_name, fn) sc = DidYouMean::SpellChecker.new(dictionary: bible_names) res = sc.correct(db_name) unless res.empty? msg = format("%s\ndid you mean %s", msg, res.join(', ')) end fail ArgumentError, msg end fn else db_name end end |
.schema_major_minor ⇒ Object
Get an array of the major and minor version of the database schema of the actual version of the lib
94 95 96 97 98 |
# File 'lib/sqlbible.rb', line 94 def schema_major_minor lib_db = SQLite3::Database.new(':memory:') lib_db.execute_batch(Sqlbible.db_schema) select_schema_major_minor(lib_db) end |
.schema_version ⇒ Object
Get the version of the database schema of the actual version of the lib
101 102 103 |
# File 'lib/sqlbible.rb', line 101 def schema_version schema_major_minor.join('.') end |
.select_schema_major_minor(db) ⇒ Object
Get an array of the major and minor version of an Sqlbible database
106 107 108 |
# File 'lib/sqlbible.rb', line 106 def select_schema_major_minor db db.execute("select major, minor from schema_version").first end |
.select_schema_version(db) ⇒ Object
Get the version of an Sqlbible database
111 112 113 |
# File 'lib/sqlbible.rb', line 111 def select_schema_version db select_schema_major_minor(db).join('.') end |
Instance Method Details
#passage(pass) ⇒ Object
Get an array of Verse instances for a Scripref::Passage
27 28 29 30 31 |
# File 'lib/sqlbible.rb', line 27 def passage pass rowid1, rowid2 = passage2rowids(pass) res = select_text(where: 'rowid between ? and ?', args: [rowid1, rowid2]) parse_verse_list(res) end |
#reference(ref) ⇒ Object
Get an array of arrays of Verse instances for a Scripref reference (array of Scripref::Passage instances)
35 36 37 |
# File 'lib/sqlbible.rb', line 35 def reference ref ref.select {|e| e.kind_of?(Scripref::Passage)}.map {|pass| passage(pass)} end |
#schema_version ⇒ Object
Get the schema version of the SQLite database of the instance
65 66 67 |
# File 'lib/sqlbible.rb', line 65 def schema_version Sqlbible.select_schema_version(@db) end |
#search(*searches, range: nil) ⇒ Object
Get an array of Verse instances for a search with one ore more regular expressions (optional a search range of a Scripref reference is possible)
41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 |
# File 'lib/sqlbible.rb', line 41 def search *searches, range: nil where_arr = [] args = [] if range passages = Array(range).select {|e| e.kind_of? Scripref::Passage} passages.size.times {where_arr << 'rowid between ? and ?'} where_arr = [or_join_where(where_arr)] args = passages.map {|p| passage2rowids(p)}.flatten end searches.flatten.each do |o| case o when Regexp where_arr << 'plaintext regexp ?' args << o.to_s else fail format('search objects of class %s are not (yet) supported', o.class) end end where = and_join_where(where_arr) res = select_text(where: where, args: args) parse_verse_list(res) end |